/*
 * Copyright 2017 NXP
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of the copyright holder nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <stdlib.h>
#include "fsl_debug_console.h"
#include "fsl_irq.h"

#include "fsl_common.h"
#include "pin_mux.h"
#include "board.h"
#include "clock_config.h"
/*******************************************************************************
 * Definitions
 ******************************************************************************/


/*******************************************************************************
 * Prototypes
 ******************************************************************************/

 /*******************************************************************************
 * Variables
 ******************************************************************************/
volatile bool g_keypress = false;

/*******************************************************************************
 * Code
 ******************************************************************************/

void IRQ_IRQHandler(void)
{
    if (IRQ_GetIRQFlag(IRQ))
    {
        /* Clear IRQ Flag. */
        IRQ_ClearIRQFlag(IRQ);
        g_keypress = true;
    }
/* Add for ARM errata 838869, affects Cortex-M4, Cortex-M4F Store immediate overlapping
  exception return operation might vector to incorrect interrupt */
#if defined __CORTEX_M && (__CORTEX_M == 4U)
    __DSB();
#endif
}

int main(void)
{
    irq_config_t irqConfig;

    /* Configure the PTA5 pin not as /RESET */
    SIM->SOPT &= ~SIM_SOPT_RSTPE_MASK;

    BOARD_InitPins();
    BOARD_BootClockRUN();
    BOARD_InitDebugConsole();

    PRINTF("\r\n IRQ Driver Example Start.\r\n");

    irqConfig.enablePullDevice = true;
    irqConfig.edgeSelect = kIRQ_FallingEdgeorLowlevel; /* Falling edge.*/
    irqConfig.detectMode = kIRQ_DetectOnEdgesOnly;
    IRQ_Init(IRQ, &irqConfig);
    IRQ_EnableInterrupt(IRQ, true);
    EnableIRQ(IRQ_IRQn);

    while (1)
    {
        if (g_keypress)
        {
            PRINTF("\r\n The IRQ interrupt has happened!\r\n");
            break;
        }
    }

    /* Deinitialize the module. */
    IRQ_Deinit(IRQ);

    PRINTF("\r\n IRQ Driver Example End.\r\n");

    while (1)
    {
    }
}
