/*
 * Copyright 2017 NXP
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of the copyright holder nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <stdlib.h>
#include "fsl_debug_console.h"
#include "fsl_kbi.h"
/*
* This case tests the necessary API to enable the ENET transmission. It would call the functions sequentially in polling
* mode.
*
*/

#include "pin_mux.h"
#include "board.h"
#include "fsl_gpio.h"
#include "clock_config.h"
/*******************************************************************************
 * Definitions
 ******************************************************************************/
#define EXAMPLE_KBI KBI0
#define EXAMPLE_KBI_SIGNAL_INPUT_REF_GPIO kGPIO_PORTA
#define EXAMPLE_KBI_SIGNAL_INPUT_REF_GPIO_INDEX 3
#define EXAMPLE_KBI_SIGNAL_INPUT_GPIO kGPIO_PORTA
#define EXAMPLE_KBI_SIGNAL_INPUT_GPIO_INDEX 2
#define EXAMPLE_KBI_PINS (2)
/*******************************************************************************
 * Prototypes
 ******************************************************************************/
/*******************************************************************************
 * Variables
 ******************************************************************************/
volatile bool g_keypress = false;
/*******************************************************************************
 * Code
 ******************************************************************************/
void KBI0_IRQHandler(void)
{
    if (KBI_IsInterruptRequestDetected(EXAMPLE_KBI))
    {
        /* Disable interrupts. */
        KBI_DisableInterrupts(EXAMPLE_KBI);
        /* Clear status. */
        KBI_ClearInterruptFlag(EXAMPLE_KBI);
        g_keypress = true;
    }
/* Add for ARM errata 838869, affects Cortex-M4, Cortex-M4F Store immediate overlapping
  exception return operation might vector to incorrect interrupt */
#if defined __CORTEX_M && (__CORTEX_M == 4U)
    __DSB();
#endif
}

int main(void)
{
    kbi_config_t kbiConfig;

    gpio_pin_config_t config = {
        kGPIO_DigitalOutput, 0,
    };  
  
    BOARD_InitPins();
    BOARD_BootClockRUN();
    BOARD_InitDebugConsole();
    
    /* Initialize for the output gpio signal. */  
    GPIO_PinInit(EXAMPLE_KBI_SIGNAL_INPUT_REF_GPIO, EXAMPLE_KBI_SIGNAL_INPUT_REF_GPIO_INDEX, &config);

    PRINTF("\r\n KBI Driver Example Start.\r\n");

    kbiConfig.mode = kKBI_EdgesDetect;
    kbiConfig.pinsEnabled = 1 << EXAMPLE_KBI_PINS;
    kbiConfig.pinsEdge = 1 << EXAMPLE_KBI_PINS; /* Raising edge.*/
    KBI_Init(EXAMPLE_KBI, &kbiConfig);

    while (1)
    {
        /* Toggle to generate the edge which will trigger the KBI interrupt. */
        GPIO_PortToggle(EXAMPLE_KBI_SIGNAL_INPUT_REF_GPIO, 1u << EXAMPLE_KBI_SIGNAL_INPUT_REF_GPIO_INDEX);

        if (g_keypress)
        {
            PRINTF("\r\n The KBI keyboard interrupt has happened!\r\n");
            break;
        }
    }

    /* Deinitialize the module. */
    KBI_Deinit(EXAMPLE_KBI);

    PRINTF("\r\n KBI Driver Example End.\r\n");

    while (1)
    {
    }
}
